@@ -2,12 +2,13 @@
 This module has utility functions for working with aws resources
 """
 import logging
-from typing import Dict, List, Sequence, Callable, Any
-import boto3
-from botocore.exceptions import ClientError, WaiterError
-from boto.exception import EC2ResponseError, BotoServerError
-from amplify_aws_utils.jitter import Jitter
+from typing import Dict, List, Sequence, Callable
 
+import boto3
+from boto.exception import EC2ResponseError, BotoServerError
+from botocore.exceptions import ClientError, WaiterError, ReadTimeoutError
+
+from amplify_aws_utils.jitter import Jitter
 from .exceptions import (
     TimeoutError,
     ExpectedTimeoutError,
@@ -91,14 +92,16 @@ def throttled_call(fun, *args, **kwargs):
                 logger.exception("Failed to run %s.", fun)
 
             error_code = err.response['Error'].get('Code', 'Unknown')
-            is_throttle_exception = any(key_word in error_code
-                                        for key_word in ("Throttling", "RequestLimitExceeded"))
+            is_throttle_exception = any(
+                key_word in error_code
+                for key_word in ("Throttling", "RequestLimitExceeded", "TooManyRequestsException")
+            )
 
             if not is_throttle_exception or time_passed > max_time:
                 raise
 
             time_passed = jitter.backoff()
-        except WaiterError:
+        except (ReadTimeoutError, WaiterError):
             if time_passed > max_time:
                 raise
 
@@ -201,9 +204,8 @@ def wait_for_sshable(remotecmd, instance, timeout=15 * 60, quiet=False):
         .format(instance, timeout))
 
 
-def get_boto3_paged_results(func, results_key, next_token_key='NextToken',
-                            next_request_token_key='NextToken', *args, **kwargs):
-    # type: (Callable, str, str, str, *Any, **Any) -> List
+def get_boto3_paged_results(func: Callable, results_key: str, next_token_key: str = 'NextToken',
+                            next_request_token_key: str = 'NextToken', *args, **kwargs) -> List:
     """
     Helper method for automatically making multiple boto3 requests for their listing functions
     :param func: Boto3 function to call
@@ -217,7 +219,7 @@ def get_boto3_paged_results(func, results_key, next_token_key='NextToken',
     if not response_items:
         logger.warning("No items found in response=%s", response)
 
-    while response.get(next_token_key):
+    while response.get(next_token_key) and response.get(results_key):
         kwargs[next_request_token_key] = response[next_token_key]
         response = throttled_call(func, *args, **kwargs)
         response_items += response[results_key]
@@ -236,8 +238,7 @@ def check_written_s3(object_name, expected_written_length, written_length):
         )
 
 
-def get_ssm_parameters(names):
-    # type: (Sequence[str]) -> Dict[str, str]
+def get_ssm_parameters(names: Sequence[str]) -> Dict[str, str]:
     """
     Convenience function for getting multiple SSM parameters
     :param names: List of names of parameters to get.
@@ -270,8 +271,25 @@ def boto3_tags_to_dict(boto3_tags):
     :param boto3_tags: List of boto3 tags.
     :return: Simple dictionary of tags
     """
+    if not boto3_tags:
+        return {}
+
+    # boto3 is not consistent with the tag dict it returns
+    # depending on the resource, the tag name will either be under a 'Key' or 'Name' key or even in lowercase
+    # check all possibilities to figure out what key names are being used
+    possible_keys = ['Key', 'key', 'Name', 'name']
+    actual_keys = (key for key in boto3_tags[0].keys() if key in possible_keys)
+    key_name = next(actual_keys, None)
+
+    possible_values = ['Value', 'value']
+    actual_values = (key for key in boto3_tags[0].keys() if key in possible_values)
+    value_name = next(actual_values, None)
+
+    if not key_name or not value_name:
+        raise RuntimeError('Unable to identify tag key names in dict')
+
     return {
-        tag['Key']: tag['Value']
+        tag[key_name]: tag[value_name]
         for tag in boto3_tags
     }
 
@@ -299,3 +317,27 @@ def chunker(sequence, size):
     # [15, 16, 17, 18, 19]
     """
     return (sequence[position:position + size] for position in range(0, len(sequence), size))
+
+
+def dynamodb_record_to_dict(record: Dict[str, Dict[str, str]]) -> Dict[str, str]:
+    """
+    Converts a DynamoDB Record into a normal Python dictionary.
+    :param record: A DynamoDB Record that looks like:
+    {
+        "foo": {
+            "S": "bar"
+        },
+        "baz": {
+            "N": "100"
+        }
+    }
+    :return: A dictionary that looks like:
+    {
+        "foo": "bar",
+        "baz": "100",
+    }
+    """
+    return {
+        key: list(value.values())[0]
+        for key, value in record.items()
+    }
